// java TCP socket demo: echo server
// server reads lines of text from socket and sends them back

import java.io.*;
import java.net.*;
import java.lang.*;

public class EchoServerMT {
    static int port = 2222;

    public static void main(String[] argv) {
        int optind = 0;
        String usage = "usage: java EchoServerMT [-p port] (default=" + port +")";

        // process command line arguments
        try {
            for (optind = 0; optind < argv.length; ++optind) {
                if (argv[optind].equals("-p")) {    // -p port
                    if (++optind >= argv.length) {
                        System.err.println(usage);
                        System.exit(1);
                    }
                    port = Integer.parseInt(argv[optind]);
                }
                else {
                    System.err.println(usage);
                    System.exit(1);
                }
            }
        } catch (NumberFormatException e) {
            System.err.println("bad number format");
            System.exit(1);
        }
        if (optind < argv.length) { // accept nothing after cmd line options
            System.err.println(usage);
            System.exit(1);
        }
        serve();
    }

    static void serve() {
        int threadID = 0;
        try {
            // the ServerSocket class allows us to create a socket,
            // bind it to a given port number for accepting TCP/IP connections
            // and set the queue length to a given value (we use 5)

            ServerSocket ss = new ServerSocket(port, 5);

            while (true) {
                System.out.println("waiting for a connection on port " + port);
                Socket s = ss.accept();    // wait for a connection
                System.out.println("got a connection from " + s.getInetAddress());

                // We want to read data a line at a time in this program.
                // A BufferedReader allows us to read characters and process
                // newline boundaries. 

                try {
                    EchoMT p = new EchoMT(s, threadID++);
                    p.start();
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                }
            }
        } catch (java.io.IOException e) {
            System.out.println(e.getMessage());
        }
    }
}
