// java TCP socket demo: echo client
// client reads lines of text from user and sends them to server
// prints return data from server - end of packet is a newline

import java.io.*;
import java.net.*;

public class EchoClient {

    static int port = 2222;       // default server port
    static String host = "localhost";      // default server host
  
    public static void main(String[] argv) {
        String usage = "usage: java EchoClient [-h host] [-p port] " +
          "\n (default port=" + port +", host=" + host + ")";

        int optind = 0;  // command-line argument index
        try {
            // process command-line arguments
            for (optind = 0; optind < argv.length; ++optind) {
                if (!argv[optind].substring(0,1).equals("-") ||
                  argv[optind].equals("--"))
                    break;    // end of options
                if (argv[optind].equals("-p")) {   // -p port
                    if (++optind >= argv.length) {
                        System.err.println(usage);
                        System.exit(1);
                    }
                    port = (new Integer(argv[optind])).intValue();
                }
                else if (argv[optind].equals("-h")) {   // -h host
                    if (++optind >= argv.length) {
                        System.err.println(usage);
                        System.exit(1);
                    }
                    host = argv[optind];
                }
                else {
                    System.err.println(usage);
                    System.exit(1);
                }
            }
        }
        catch (NumberFormatException e) {
            System.err.println("bad number format");
            System.err.println(usage);
            System.exit(1);
        }

        if (optind < argv.length) { // nothing after cmd line options
            System.err.println(usage);
            System.exit(1);
        }
        echo();
    }

    // connect to the server, read lines of text from user, send them over
    // and read response.
 
    static void echo() {
        try {
            // this creates a TCP/IP socket to the machine <host> on port <port>
            Socket s = new Socket(host, port);

            // we have a connection --
            // now get the Input and Output streams
            // layer a BufferedReader on the InputStream so we can read lines
            // layer a PrintStream on the OutputStream so we can do println
            PrintStream netout = new PrintStream(s.getOutputStream());
            BufferedReader netin = new BufferedReader(
                                new InputStreamReader(s.getInputStream()));
            // console reader
            BufferedReader in = new BufferedReader(
                                new InputStreamReader(System.in));

            // loop until eof, reading text from the standard input
            // and sending it to the network.
            // then read the response from the network and
            // print it to the standard output

            String intext;
            while ((intext = in.readLine()) != null) {
                netout.println(intext);
                    
                String response = netin.readLine();
                if (response == null)
                    break;  // server closed connection 
                System.out.println(response);
            }
            // no more data, close connection
            s.close();
        }
        catch (UnknownHostException e) {
            System.err.println("unknown host: " + host);
            System.exit(1);
        }
        catch (IOException e) {
            System.out.println(e.getMessage());
        }
    }
}
