// java TCP socket demo: echo server
// server reads lines of text from socket and sends them back

import java.io.*;
import java.net.*;
import java.lang.*;

public class EchoServer {
    static int port = 2222;

    public static void main(String[] argv) {
        int optind = 0;
        String usage = "usage: java EchoServer [-p port] (default=" + port +")";

        // process command line arguments
        try {
            for (optind = 0; optind < argv.length; ++optind) {
                if (argv[optind].equals("-p")) {    // -p port
                    if (++optind >= argv.length) {
                        System.err.println(usage);
                        System.exit(1);
                    }
                    port = Integer.parseInt(argv[optind]);
                }
                else {
                    System.err.println(usage);
                    System.exit(1);
                }
            }
        } catch (NumberFormatException e) {
            System.err.println("bad number format");
            System.exit(1);
        }
        if (optind < argv.length) { // accept nothing after cmd line options
            System.err.println(usage);
            System.exit(1);
        }
        serve();
    }

    static void serve() {
        try {
            // the ServerSocket class allows us to create a socket,
            // bind it to a given port number for accepting TCP/IP connections
            // and set the queue length to a given value (we use 5)

            ServerSocket ss = new ServerSocket(port, 5);

            while (true) {
                System.out.println("waiting for a connection on port " + port);
                Socket s = ss.accept();    // wait for a connection
                System.out.println("got a connection from " + s.getInetAddress());

                // We want to read data a line at a time in this program.
                // A BufferedReader allows us to read characters and process
                // newline boundaries. 

                try {
                    BufferedReader in = new BufferedReader(
                                    new InputStreamReader(s.getInputStream()));
                    PrintStream out = new PrintStream(s.getOutputStream());
                    String line;
                    while ((line =in.readLine()) != null) {
                        System.out.println("received: \"" + line + "\"");
                        // allow a "." to terminate this session
                        if (line.equals("."))
                            break;
                        out.println(line);
                    }
                    s.close();
                } catch (java.io.IOException e) {
                    System.out.println(e.getMessage());
                }
            }
        } catch (java.io.IOException e) {
            System.out.println(e.getMessage());
        }
    }
}
