/*
	info: tiny demo of sockets programming
 	      this one is connection-oriented

	usage:	info [-d] [-h serverhost] [-p port] [cmd ...]

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>      /* for gethostbyname() */
#include <unistd.h>     /* for isatty() */
#include <sys/socket.h>
#include <netinet/in.h>

int debug = 0;
int conn(char *host, int port);
void disconn();
void cmd(char *s);      /* process remote commands */

main(int argc, char **argv)
{
	int fd, nbytes;
	int port;
	char *host;
	char *usage = "usage: client server_host server_port\n";
	char buf[BUFSIZ];

	if (argc != 3) {
		fprintf(stderr, usage);
		exit(1);
	}
	host = argv[1];
	port = atoi(argv[2]);
	if (port <= 0 || port > 65535) {
		fprintf(stderr, "invalid port number: %s\n", argv[2]);
		fprintf(stderr, usage);
		exit(1);
	}

	if ((fd = conn(host, port)) == -1)    /* connect */
		exit(1);   /* something went wrong */

	/* now we can read and write on the file descriptor */
	/* copy stdin to the socket */
	while ((nbytes=read(0, buf, BUFSIZ)) > 0) {
		if (write(fd, buf, nbytes) != nbytes) {
			fprintf(stderr, "write error\n");
			break;
		}
	}
	fprintf(stderr, "done\n");
	disconn();    /* disconnect */
	return 0;
}


/* conn: connect to the service running on host:port */
/* return socket descriptor on success, -1 on failure */
int
conn(char *host, int port)
{
	struct hostent *hp;	/* host information */
	struct sockaddr_in myaddr;	/* our address */
	struct sockaddr_in servaddr;	/* server address */
	int fd;  /* socket descriptor */

	/* get a tcp/ip socket */
	/* request the Internet address protocol */
	/* and a reliable 2-way byte stream */

	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("cannot create socket");
		return 0;
	}

	/* bind to an arbitrary return address */
	/* because this is the client side, we don't care about the address */
	/* no application will connect here  --- */
	/* INADDR_ANY is the IP address and 0 is the socket */
	/* htonl converts a long integer (e.g. address) to a network */
	/* representation (agreed-upon byte ordering */

	memset((char *)&myaddr, 0, sizeof(myaddr));
	myaddr.sin_family = AF_INET;
	myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	myaddr.sin_port = htons(0);

	if (bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
		perror("bind failed");
		return -1;
	}

	/* fill in the server's address and data */
	/* htons() converts a short integer to a network representation */

	memset((char*)&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(port);

	/* look up the address of the server given its name */
	hp = gethostbyname(host);
	if (!hp) {
		fprintf(stderr, "could not obtain address of %s\n", host);
		return -1;
	}

	/* put the host's address into the server address structure */
	memcpy((void *)&servaddr.sin_addr, hp->h_addr_list[0], hp->h_length);

	/* connect to server */
	if (connect(fd, (struct sockaddr *)&servaddr, sizeof(servaddr)) < 0) {
		perror("connect failed");
		return -1;
	}
	return fd;
}

/* disconn from the service */
void
disconn(int fd)
{
	shutdown(fd, 2);    /* 2 means future sends & receives are disallowed */
}
