// this is a demo of a most rudimentary java based server 
// it just accepts a single socket connection and reads
// a stream of data, printing it to the standard output

import java.io.*;
import java.net.*;
import java.lang.*;

public class Server {

	static int port;       // server port

	public static void main(String[] arg) {
		String usage = "usage: java Server port\n";

		if (arg.length != 1) {
			System.err.println(usage);
			System.exit(1);
		}
		try {
			port = (new Integer(arg[0])).intValue();
		} catch (NumberFormatException e) {
			System.err.println("bad number format: " + arg[2]);
			System.err.println(usage);
			System.exit(1);
		}
		serve();
	}

	// create socket and wait for connections
	public static void serve() {
		try {
			final int bufsize = 4096;
			byte buf[] = new byte[bufsize];
			int nbytes;

			// the ServerSocket class allows us to create a socket,
			// bind it to a given port number for accepting TCP/IP connections
			// and set the queue length to a reasonable value (we use 5)

			ServerSocket ss = new ServerSocket(port, 5);

			while (true) {
				System.out.println("waiting for a connection on port " + port);
				Socket s = ss.accept();    // wait for a connection
				System.out.println("got a connection from "
				    + s.getInetAddress().getHostAddress()
				    + ", port: " + s.getPort());
				// grab the input stream and read data from it until
				// the client closes the connection

				InputStream in = s.getInputStream();
				while ((nbytes = in.read(buf)) != -1)
					System.out.write(buf, 0, nbytes);
				// close the stream on the socket
				in.close();
				s.close();
			}
		} catch (java.io.IOException e) {
			System.out.println(e.getMessage());
		}
	}
}
