/*
	server  a tiny demo of sockets programming
	  this one is connectionless
	  single threaded, single process version:
	  	only one client request at a time.

	usage:	server
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>

extern int errno;

void serve(int port);	/* main server function */
char *ipfmt(unsigned long a);	/* format address to printable format */

main(int argc, char **argv)
{
	char *usage = "usage: server port\n";
	int port;

	if (argc != 2) {
		fprintf(stderr, usage);
		exit(1);
	}
	port = atoi(argv[1]);
	if (port <= 0 || port > 65535) {
		fprintf(stderr, "invalid port: %s\n", argv[1]);
		fprintf(stderr, usage);
		exit(1);
	}
	serve(port);
}

/* serve: set up the service */

void
serve(int port)
{
	int svc;        /* listening socket providing service */
	int rqst;       /* socket accepting the request */
	socklen_t alen;       /* length of address structure */
	int nbytes;
	struct sockaddr_in my_addr;    /* address of this service */
	struct sockaddr_in client_addr;  /* client's address */
	int sockoptval = 1;
	char hostname[128]; /* host name, for debugging */

	gethostname(hostname, 128);

	/* get a tcp/ip socket */
	/*   AF_INET is the Internet address (protocol) family  */
	/*   with SOCK_DGRAM we ask for a datagram socket */
	/*   With IP, this means that TCP will be used */

	if ((svc = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("cannot create socket");
		exit(1);
	}

	/* we use setsockopt to set SO_REUSEADDR. This allows us */
	/* to reuse the port immediately as soon as the service exits. */
	/* Some operating systems will not allow immediate reuse */
	/* on the chance that some packets may still be en route */
	/* to the port. */

	setsockopt(svc, SOL_SOCKET, SO_REUSEADDR, &sockoptval, sizeof(int));

	/* set up our address */
	/* htons converts a short integer into the network representation */
	/* htonl converts a long integer into the network representation */
	/* INADDR_ANY is the special IP address 0.0.0.0 which binds the */
	/* transport endpoint to all IP addresses on the machine. */

	memset((char*)&my_addr, 0, sizeof(my_addr));  /* 0 out the structure */
	my_addr.sin_family = AF_INET;   /* address family */
	my_addr.sin_port = htons(port);
	my_addr.sin_addr.s_addr = htonl(INADDR_ANY);

	/* bind to the address to which the service will be offered */

	if (bind(svc, (struct sockaddr *)&my_addr, sizeof(my_addr)) < 0) {
		perror("bind failed");
		exit(1);
	}

	printf("server started on %s, listening on port %d\n", hostname, port);

	alen = sizeof(client_addr);     /* length of address */
	printf("addr size = %d bytes\n", alen);

	for (;;) {
		const int bufsize=1024;
		char buf[bufsize];
		int flags = 0;

		while ((nbytes = recvfrom(svc, buf, bufsize, flags, (struct sockaddr *)&client_addr, &alen)) >= 0) {
			printf("received %d byte request from %s port %d: %d bytes\n",
				alen,
				ipfmt(ntohl(client_addr.sin_addr.s_addr)),
				ntohs(client_addr.sin_port), nbytes);

			/* dostuff(rqst); run the service */
		}
		printf("received %d from client\n", nbytes);
		close(rqst);
	}
}

/* dostuff: process client requests */
/* here we just read from the client until the client closes the connection */
/* all data is printed to the standard output */

dostuff(int fd) {
	char buf[BUFSIZ];
	int n;

	while ((n=read(fd, buf, BUFSIZ)) > 0)
		write(1, buf, n);
}

/* format ip address to dot-separated text format */

char *
ipfmt(unsigned long a) {
	static char p[16];
	sprintf(p, "%u.%u.%u.%u", (a>>24)&0xff, (a>>16)&0xff,
	        (a>>8)&0xff, a&0xff);
	return p;
}
